/* written by John Doug Reynolds, August 2001 */

#include "rgsevconvert.h"
#include <Dal.h>
#include <cstdio>

void rgsevconvert ()
{
   string dsname = stringParameter("set");
   struct DataSet *dataset = dataSetServer->open( dsname, Dal::Modify);

   // convert the EVENTS table

   struct Table *table = dataset->table("EVENTS");

   if ( table->hasAttribute("PI_REF") ) {
      struct Column *chc = table->column("PI");
      *chc->addAttribute("TCTYP","[PI] channel axis type") = "";
      *chc->addAttribute("TCUNI","channel axis units") = "PI";
      *chc->addAttribute("TCRPX","reference channel number") = 0;
      *chc->addAttribute("TCRVL","center of reference channel")
	 = table->attribute("PI_REF")->asReal();
      *chc->addAttribute("TCDLT","uniform channel width")
	 = table->attribute("PI_WID")->asReal();
      table->deleteAttribute("PI_REF");
      table->deleteAttribute("PI_WID");
   }

   if ( table->hasAttribute("BETA_REF") ) {
      struct Column *chc = table->column("BETA_CHANNEL");
      *chc->addAttribute("TCTYP","[dispersion] channel axis type") = "";
      *chc->addAttribute("TCUNI","channel axis units") = "rad";
      *chc->addAttribute("TCRPX","reference channel number") = 0;
      *chc->addAttribute("TCRVL","center of reference channel")
	 = table->attribute("BETA_REF")->asReal();
      *chc->addAttribute("TCDLT","uniform channel width")
	 = table->attribute("BETA_WID")->asReal();
      table->deleteAttribute("BETA_REF");
      table->deleteAttribute("BETA_WID");
   }

   if ( table->hasAttribute("XDSP_REF") ) {
      struct Column *chc = table->column("XDSP_CHANNEL");
      *chc->addAttribute("TCTYP","[cross-dispersion] channel axis type") = "";
      *chc->addAttribute("TCUNI","channel axis units") = "rad";
      *chc->addAttribute("TCRPX","reference channel number") = 0;
      *chc->addAttribute("TCRVL","center of reference channel")
	 = table->attribute("XDSP_REF")->asReal();
      *chc->addAttribute("TCDLT","uniform channel width")
	 = table->attribute("XDSP_WID")->asReal();
      table->deleteAttribute("XDSP_REF");
      table->deleteAttribute("XDSP_WID");
   }

   // convert the EXPMn_0c images

   for ( unsigned ccd = 1; ccd <= 9; ++ccd )
      for ( unsigned node = 0; node < 2; ++node ) {
	 char extname[9];
	 sprintf( extname, "EXPM%d_0%d", node, ccd );
	 if ( !dataset->hasBlock(extname) ) continue;

	 struct Array *array = dataset->array(extname);
	 sprintf( extname, "EXPMAP%d%d", node, ccd );
	 array->rename(extname);

	 array->attribute("NODEID")->rename("CCDNODE");

	 *array->addAttribute("CTYPE1","[dispersion] channel axis type") = "";
	 *array->addAttribute("CUNIT1","channel axis units") = "rad";
	 *array->addAttribute("CRPIX1","reference channel coordinate") = 1;
	 *array->addAttribute("CRVAL1","reference physical coordinate")
	    = array->attribute("BETA_REF")->asReal();
	 *array->addAttribute("CDELT1","uniform channel width")
	    = array->attribute("BETA_WID")->asReal();
	 array->deleteAttribute("BETA_REF");
	 array->deleteAttribute("BETA_WID");

	 *array->addAttribute("CTYPE2","[cross-dispersion] channel axis type")
	    = "";
	 *array->addAttribute("CUNIT2","channel axis units") = "rad";
	 *array->addAttribute("CRPIX2","reference channel coordinate") = 1;
	 *array->addAttribute("CRVAL2","reference physical coordinate")
	    = array->attribute("XDSP_REF")->asReal();
	 *array->addAttribute("CDELT2","uniform channel width")
	    = array->attribute("XDSP_WID")->asReal();
	 array->deleteAttribute("XDSP_REF");
	 array->deleteAttribute("XDSP_WID");
      }

   //

   dataSetServer->close( dataset );
}
